;
;Control program for HRPT receiver with 35MHz IF
;modified for 16f84 PIC processor with Meteosat 7 frequencies added by GWR
;

;************************************************************************
; processor pic16f84                                                    *
; no code protection                                                    *
; power up timer enabled                                                *
; WDT disabled; gwr                                                     *
; RC oscillator  (? MHz)                                             *
;************************************************************************
;***** Declaring and configuring a microcontroller *****

        PROCESSOR 16f84
        #include "p16f84.inc"
	
	__CONFIG _CP_OFF & _WDT_OFF & _PWRTE_ON & _RC_OSC


;************************************************************************

        #include "bank.inc"	; Assistant files

;************************************************************************

;
;Define registers
;Variables start after 0x0c

#define synpoint        H'00d'          ;points to current synth data
#define datalen         H'00e'          ;how many bits in current data sequence
#define chanstr         H'00f'          ;previous channel selection
#define chansel         H'010'          ;current channel selection on switch or external interface
#define bitctr          H'011'          ;counts off bits in byte
#define wordctr         H'012'          ;counts off synth data words
#define charbuff        H'013'          ;store for synth data characters

;
;Define synth connections
;

#define	syndata		PORTA,1		;data
#define	synclk		PORTA,2		;clock
#define	synle		PORTA,0		;load enable
#define upbank          PORTA,3         ;upper bank when high: HRPT or CHRPT
#define upmem           PORTA,4         ;PDUS or CHRPT when high
;

;
;Define valid input values for channel selections
;

chan1		equ	D'126'		;01111110
chan2		equ	D'125'		;01111101
chan3		equ	D'123'		;01111011
chan4		equ	D'119'		;01110111
chan5		equ	D'111'		;01101111
chan6		equ	D'95'		;01011111
chan7		equ	D'63'		;00111111



;
;************************************************************************

	ORG     0x00	; Reset vector
  	goto    start

	ORG     0x04	; Interrupt vector
        goto    start   ; No interrupt routine

;************************************************************************



;
;Program starts here
;
;
;Configure port A for input
;

start
	BANK1
        movlw   0xf0            ; output on PortA 0,1,2,3
	movwf	TRISA
	BANK0
        clrf    PORTA

;
;Reset the synth and set to chan 6 (1707MHz)
;

		;
		;First put synth in reset mode
		;

		movlw	D'20'		;set data length count
		movwf	datalen
		movlw	reset1-table	;point to reset1 data
		movwf	synpoint
		call	synthdata

		;
		;Then set the reference divider
		;

		movlw	D'21'
		movwf	datalen
		movlw	rcount-table	;point to R counter data
		movwf	synpoint
		call 	synthdata

		;
		;Then set the default channel 1707MHz
		;

		movlw	D'21'
		movwf	datalen
		movlw	ch6-table	;point to channel 6 data
		movwf	synpoint
		call	synthdata

		;
		;Return from reset mode
		;

		movlw	D'20'
		movwf	datalen
		movlw	reset2-table
		movwf	synpoint
		call 	synthdata

		movlw	0		;set initial channel store value
		movwf	chanstr

;
;Enter the main program loop
;Scan port A and look for change of input value.
;If any change, re-program the synth.
;

loop		movf	PORTB,0		;get channel select data to W
		andlw	H'7F'		;mask off msb
		movwf	chansel		;store channel selection
		xorwf	chanstr,0	;check for channel change
		btfsc	STATUS,Z
		goto	loop


                clrf    PORTA           ;default decoder to HRPT in lower bank
                movf    chansel,0       ;store the changed channel
		movwf	chanstr

		xorlw	chan1		;check for channel 1
		btfss	STATUS,Z
		goto	loop1		;go if not chan 1..
		movlw	ch1-table	;else point to chan 1 data
		goto	loop7

loop1		movf	chanstr,0	;get channel to W
		xorlw	chan2		;check for channel 2
		btfss	STATUS,Z
		goto	loop2		;go if not chan 2..
		movlw	ch2-table	;else point to chan2 data
		goto	loop7

loop2		movf	chanstr,0	;get channel to W
		xorlw	chan3		;check for channel 3
		btfss	STATUS,Z
		goto	loop3		;go if not chan 3..
                bsf     PORTA,3         ;switch alblas decoder to chrpt
                bsf     PORTA,4         ;in upper bank & upper mem
		movlw	ch3-table	;else point to chan3 data
		goto	loop7

loop3		movf	chanstr,0	;get channel to W
		xorlw	chan4		;check for channel 4	
		btfss	STATUS,Z
		goto	loop4		;go if not chan 4..
                movlw   ch4-table       ;else point to chan4 data
		goto	loop7

loop4		movf	chanstr,0	;get channel to W
		xorlw	chan5		;check for channel 5
		btfss	STATUS,Z
		goto	loop5		;go if not chan 5..
                bsf     PORTA,4         ;PDUS in lower bank & upper mem
                movlw   ch5-table       ;else point to chan5 data
		goto	loop7

loop5		movf	chanstr,0	;get channel to W
		xorlw	chan6		;check for channel 6
		btfss	STATUS,Z
		goto	loop6		;go if not chan 5..
                bsf     PORTA,4         ;PDUS in lower bank & upper mem
                movlw   ch6-table       ;else point to chan6 data
		goto	loop7

loop6		movlw	ch7-table	;else point to chan7 data

loop7		movwf	synpoint	;store table pointer
		movlw	D'21'		;set data length count
		movwf	datalen
		call	synthdata
		goto	loop

;
;Send channel or reset data to the synthesizer.
;Following code is entered with synpoint and datalen set.
;

synthdata	bcf	synclk		;clear clock bit
		bcf	synle		;clear load enable

synthdata1	movlw	D'8'		;set bit counter
		movwf	bitctr
		movf	synpoint,0	;put table offset pointer in W
		call	getdata		;get character from table
		movwf	charbuff	;temp store data byte
		incf	synpoint,1	;point to next data byte

synthdata2	btfsc	charbuff,7	;check if msb clear..
		goto	synthdata3	;if not go and set data bit..
		bcf	syndata		;else clear data bit..
		goto	synthdata4	;and then send data bit

synthdata3	bsf	syndata		;set data bit

synthdata4	nop
		bsf	synclk		;pulse clock high..
		bcf	synclk		;and then low
		rlf	charbuff,1	;move to next data bit
		decf	datalen,1	;dec the length count..
		decfsz	bitctr,1	;and the bit/byte count
					
		goto	synthdata5	;if bit/byte count not zero..
					;go test the length count..
		goto	synthdata1	;else get the next byte

synthdata5	movf	datalen,1	;test length count using Z
		btfss	STATUS,Z	;if not zero..
		goto	synthdata2	;process next bit..
		bsf	synle		;else send synth LE
		bcf	synle
		retlw	0
		
;	
;Data tables
;		

getdata		addwf	PCL,1		;offset into table
table
reset1		retlw	0		;data to enter reset mode
		retlw	D'8'
		retlw	D'96'

reset2		retlw	0		;data to leave reset mode
		retlw	D'8'
		retlw	D'32'

rcount		retlw	0		;reference counter data
		retlw	D'12'
		retlw	D'128'

;
;Frequency settings for 35Mhz IF
;Frequency in comments is actual receive frequency.  The data programmes
;the synth to 35MHz below the receive frequency.
;

ch1		retlw	D'8'		;1707		HRPT
		retlw	D'42'
		retlw	D'8'

ch2		retlw	D'8'		;1698		HRPT
		retlw	D'30'
		retlw	D'200'

ch3		retlw	D'8'		;1700.4		CHRPT
		retlw	D'33'
		retlw	D'200'

ch4		retlw	D'8'		;1702.5		HRPT
		retlw	D'36'
		retlw	D'104'

ch5		retlw	D'8'		;1694.5		PDUS2
		retlw	D'26'
		retlw	D'104'

ch6		retlw	D'8'		;1691		PDUS1
		retlw	D'22'
		retlw	D'8'

ch7		retlw	D'8'		;1707		HRPT
		retlw	D'42'
		retlw	D'8'

	End		; End of program






